// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "prefab/SceneItemPrefab.hpp"
#include "asset/AssetManager.hpp"
#include "poker/PokerPlayer.hpp"
#include "scene/components/Components.hpp"
#include "visualnovel/components/VisualNovelCharacter.hpp"
#include "display/animation/TiledSpriteAnimation.hpp"

namespace Dawn {
  class PennyPrefab : public SceneItemPrefab<PennyPrefab> {
    public:
      VisualNovelCharacter *vnCharacter;

      static std::vector<Asset*> prefabAssets(AssetManager *assMan) {
        return std::vector<Asset*>{
          assMan->get<TextureAsset>("texture_penny"),
          assMan->get<TilesetAsset>("tileset_penny")
        };
      }

      PennyPrefab(Scene *scene, sceneitemid_t id) : SceneItemPrefab(scene, id){}

      void prefabInit(AssetManager *man) override {
        auto textureAsset = man->get<TextureAsset>("texture_penny");
        auto tilesetAsset = man->get<TilesetAsset>("tileset_penny");

        auto meshRenderer = this->addComponent<MeshRenderer>();
        auto material = this->addComponent<Material>();
        auto meshHost = this->addComponent<MeshHost>();
        auto tiledSprite = this->addComponent<TiledSprite>();
        auto animation = this->addComponent<AnimationController>();
        auto pokerPlayer = this->addComponent<PokerPlayer>();
        vnCharacter = this->addComponent<VisualNovelCharacter>();
        
        vnCharacter->nameKey = "character.penny.name";

        auto param = material->getShader()->getParameterByName("u_Text");
        material->textureValues[param] = &textureAsset->texture;
        
        tiledSprite->setTilesetAndSize(&tilesetAsset->tileset);
        tiledSprite->setTile(0);
        this->transform.setLocalPosition(glm::vec3(0, 0, 0));

        auto anim = new TiledSpriteAnimation(tiledSprite);
        anim->addSequentialKeyframes(0.1f, 0, 22);
        anim->loop = true;
        animation->animation = anim;
      }
  };
}