// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scenes/PixelVNScene.hpp"
#include "scenes/Scene_8.hpp"

namespace Dawn {
  class Scene_7 : public PixelVNScene {
    protected:
      void vnStage() override {
        PixelVNScene::vnStage();
      }

      void onSceneEnded() {
        auto scene = new SubSceneRendererScene<Scene_8>(this->game);
        auto assets = game->scene->getRequiredAssets();
        game->assetManager.queueSwap(assets, this->getRequiredAssets());
        game->assetManager.syncLoad();
        scene->stage();
        this->game->sceneCutover(scene);
      }

      IVisualNovelEvent * getVNEvent() override {
        auto start = new VisualNovelPauseEvent(vnManager, 0.1f);

        start
          ->then(new VisualNovelTextboxEvent(vnManager, nullptr, "scene.7.1"))
          ->then(new VisualNovelCallbackEvent<Scene_7>(vnManager, this, &Scene_7::onSceneEnded))
        ;

        return start;
      }

    public:
      Scene_7(DawnGame *game) : PixelVNScene(game) {
      }

      std::vector<Asset*> getRequiredAssets() override {
        std::vector<Asset*> assets = PixelVNScene::getRequiredAssets();
        return assets;
      }
  };
}