// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/Scene.hpp"
#include "game/DawnGame.hpp"

namespace Dawn {
  template<class T>
  class SubSceneRendererScene : public Scene {
    public:
      Camera *camera;
      SceneItem *sceneItem;
      T subScene;

      SubSceneRendererScene(DawnGame *game) : Scene(game), subScene(game) {

      }

      std::vector<Asset*> getRequiredAssets() override {
        return this->subScene.getRequiredAssets();
      }

      void stage() override {
        // Stage subscene
        this->subScene.stage();

        // Create camera to render.
        this->camera = Camera::create(this);
        this->camera->getRenderTarget()->setClearColor(COLOR_BLACK);

        // Add render quad.
        this->sceneItem = this->createSceneItem();
        auto host = this->sceneItem->addComponent<MeshHost>();
        auto renderer = this->sceneItem->addComponent<MeshRenderer>();

        auto material = this->sceneItem->addComponent<Material>();
        material->textureValues[material->getShader()->getParameterByName("u_Text")] = this->subScene.renderTarget.getTexture();

        auto renderTargetQuad = this->sceneItem->addComponent<SimpleRenderTargetQuad>();
        renderTargetQuad->setRenderTarget(&this->subScene.renderTarget);
        
        auto subSceneController = this->sceneItem->addComponent<SubSceneController>();
        subSceneController->setSubScene(&this->subScene);

        auto subSceneCameraAlign = this->sceneItem->addComponent<SubSceneCameraAlign>();
        subSceneCameraAlign->setRenderTarget(&this->subScene.renderTarget);
        subSceneCameraAlign->setCamera(this->camera);
      }
  };
}