/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "common.h"

#define FILE_CHILD_TYPE_DIR 0x00
#define FILE_CHILD_TYPE_FILE 0x01
#define FILE_CHILD_NAME_MAX 512
#define FILE_CHILD_COUNT_MAX 64

#if defined(_MSC_VER)
  #include <direct.h>
  #include <windows.h>
  #define getcwd _getcwd
  #define FILE_PATH_SEP '\\'
  #define fileMkdir(path, perms) _mkdir(path)
#elif defined(__GNUC__)
  #include <unistd.h>
  #include <dirent.h>
  #include <sys/stat.h>
  #define FILE_PATH_SEP '/'
  #define fileMkdir(path, perms) mkdir(path, perms)
#endif

void fileNormalizeSlashes(char *string);

void fileMkdirp(char *path);

size_t assetReadString(FILE *file, char *buffer);

void fileGetDirectory(char *file, char* buffer);

bool fileListChildren(
  char *directory,
  char *buffer,
  int32_t *count,
  uint8_t *types,
  char **children
);

/**
 * Reads ahead to the first instance of the given character you provide.
 * 
 * @param bufferIn Buffer to scan.
 * @param start Start position within the buffer to scan from (inclusive).
 * @param bufferOut Where to write the temporary data that was read ahead.
 * @param needles Array of characters to scan for.
 * @param needleCount How many elements are within the needles array.
 * @return The count of characters skipped.
 */
int32_t readAhead(
  char *bufferIn, int32_t start,
  char *bufferOut,
  char *needles, int32_t needleCount
);


/**
 * Skips any characters found in the needles.
 * 
 * @param bufferIn Buffer of chars to read.
 * @param start Start of the buffer.
 * @param needles Needles you are trying to skip.
 * @param needleCount Count of needles in the needles array.
 * @return The count of chars to skip ahead.
 */
int32_t skipAhead(
  char *bufferIn, int32_t start, 
  char *needles, int32_t needleCount
);